@@ -8,6 +8,7 @@
 
 
 local t = {
+   version = 3,
    dir = 'tate',
    zw = 1.0, zh = 1.0,
    kanjiskip =  { 0.0, 0.25, 0 },
@@ -43,6 +44,7 @@ local t = {
 	 [3] = { 0.25, 0.0, 0.25, priority=1 },
 	 [4] = { 0, 0, 0, kanjiskip_shrink=1 },
 	 [5] = { 0, 0, 0, kanjiskip_shrink=1 },
+	 [105] = { 0, 0, 0, kanjiskip_shrink=1 },
 	 [6] = { 0, 0, 0, kanjiskip_shrink=1 },
 	 [7] = { 0, 0, 0, kanjiskip_shrink=1 },
 	 [107] = { 0, 0, 0, kanjiskip_shrink=1 },
@@ -65,12 +67,13 @@ local t = {
          [2] = { 0, 0, 0, kanjiskip_shrink=1 },
 	 [3] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
          [4] = { 0, 0, 0, kanjiskip_shrink=1 },
-	 [5] = { 0.5 , 0.0, 0.5, rario=0, kanjiskip_stretch=1 },
-	 [6] = { 0.5 , 0.0, 0.5, rario=0, kanjiskip_stretch=1 },
-	 [7] = { 0.5 , 0.0, 0.5, rario=0, kanjiskip_stretch=1 },
-	 [107] = { 0.5 , 0.0, 0.5, rario=0, kanjiskip_stretch=1 },
-	 [207] = { 0.5 , 0.0, 0.5, rario=0, kanjiskip_stretch=1 },
-	 [8] = { 0.5 , 0.0, 0.5, rario=0, kanjiskip_stretch=1 },
+	 [5] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
+	 [105] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
+	 [6] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
+	 [7] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
+	 [107] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
+	 [207] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
+	 [8] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
       }
    },
 
@@ -87,6 +90,7 @@ local t = {
 	 [3] = { 0.5 , 0.0, 0.25, priority=1 },
 	 [4] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
 	 [5] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
+	 [105] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
 	 [6] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
 	 [7] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
 	 [107] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
@@ -105,6 +109,7 @@ local t = {
 	 [1] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
 	 [3] = { 0.75, 0.0, 0.25, priority=1, ratio=1./3, kanjiskip_stretch=1 },
 	 [5] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
+	 [105] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
 	 [6] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
 	 [7] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
 	 [107] = { 0.5 , 0.0, 0.5, ratio=0, kanjiskip_stretch=1 },
@@ -113,7 +118,7 @@ local t = {
       }
    },
 
-   [5] = { -- ダッシュ
+   [5] = { -- 分離禁止文字
       chars = { '—', '―', '‥', '…', '〳', '〴', '〵', },
       align = 'left', left = 0.0, down = 0.0,
       width = 1.0, height = 0.50, depth = 0.50, italic=0.0,
@@ -125,7 +130,25 @@ local t = {
 	 [6] = { 0, 0, 0, kanjiskip_shrink=1 },
       },
       kern = {
-	 [5] = 0.0
+         [5] = 0.0,
+         [105] = 0.0,
+      }
+   },
+
+   [5] = { -- -- 二分（二重）ダッシュ
+      chars = { '゠', '–' },
+      align = 'middle', left = 0.0, down = 0.0,
+      width = 0.50, height = 0.50, depth = 0.50, italic=0.0,
+      glue = {
+	 [1] = { 0.5 , 0.0, 0.5, ratio=1, kanjiskip_stretch=1 },
+	 [2] = { 0, 0, 0, kanjiskip_shrink=1 },
+	 [3] = { 0.25, 0.0, 0.25, priority=1, ratio=1 },
+	 [4] = { 0, 0, 0, kanjiskip_shrink=1 },
+	 [6] = { 0, 0, 0, kanjiskip_shrink=1 },
+      },
+      kern = {
+	 [5] = 0.0,
+	 [105] = 0.0
       }
    },
 
@@ -146,7 +169,8 @@ local t = {
 	 [8] = { 0, 0, 0, kanjiskip_shrink=1 },
       },
       kern = {
-	 [5] = 0.0
+	 [5] = 0.0,
+	 [105] = 0.0,
       }
    },
 
@@ -236,10 +260,7 @@ local t = {
    },
 
    [99] = { -- box末尾
-      chars = {'boxbdd', 'parbdd'},
-      glue = {
-	 [3] = { 0.25, 0.0, 0.25, priority=1 },
-      }
+      chars = {'boxbdd', 'parbdd', 'glue'},
    },
 
 }
@@ -251,5 +272,13 @@ for i=13274,13294 do ht[#ht+1] = 'AJ1-' .. tostring(i) end
 local ht = t[207].chars
 for i=13254,13273 do ht[#ht+1] = 'AJ1-' .. tostring(i) end
 
+t[100]=table.fastcopy(t[0])
+t[100].chars={'nox_alchar'}
+for i,v in pairs(t) do
+  if i~=6 and type(i)=='number' and type(v)=='table' then -- 感嘆符以外
+    if v.glue and v.glue[0] then v.glue[100] = v.glue[0] end
+    if v.kern and v.kern[0] then v.kern[100] = v.kern[0] end
+  end
+end
 
 luatexja.jfont.define_jfm(t)
